"""Request search_runner class."""

import requests

from . import WebClient
from .request_details import RequestDetails
from .request_response import RequestResponse


class WebClientRequests(WebClient):
    """Request search_runner class. Implementation based on requests library."""

    def run_request(self, params: RequestDetails) -> RequestResponse:
        """Main method to run request using requests package."""
        session = requests.Session()
        response = session.request(
            method=params.http_method.name,
            url=params.url,
            params=params.params,
            headers=params.headers,
            timeout=params.timeout
        )
        return RequestResponse(response.status_code, response.text)



"""Util to process access token to Twitter api."""

import re

from .auth_token_provider import AuthTokenProvider, AuthTokenProviderFactory
from ..exceptions import RefreshTokenException
from ..http_request import HttpMethod
from ..http_request import RequestDetails, WebClient

_retries = 5
_timeout = 20
_url = 'https://twitter.com'


class TwitterAuthTokenProvider(AuthTokenProvider):
    """Class to manage Twitter token api."""

    web_client: WebClient

    def __init__(self, web_client: WebClient):
        """Constructor of TokenRequest."""
        self.web_client = web_client
        return

    def _request_for_response_body(self):
        """Method from Twint."""
        token_request_details = RequestDetails(HttpMethod.GET, _url, dict(), dict(), _timeout)
        token_response = self.web_client.run_request(token_request_details)
        if token_response.is_success():
            return token_response.text
        else:
            raise RefreshTokenException('Error during request for token')

    def get_new_token(self) -> str:
        """Method to get refreshed token. In case of error raise RefreshTokenException."""
        token_html = self._request_for_response_body()
        match = re.search(r'\("gt=(\d+);', token_html)
        if match:
            return str(match.group(1))
        else:
            print('Could not find the Guest token in HTML')
            print(token_html)
            raise RefreshTokenException('Could not find the Guest token in HTML')


class TwitterAuthTokenProviderFactory(AuthTokenProviderFactory):
    """Provider of TwitterAuthTokenProviderFactory."""

    def create(self, web_client: WebClient) -> AuthTokenProvider:
        """Method to create TwitterAuthTokenProvider from web_client."""
        return TwitterAuthTokenProvider(web_client)



from .auth_token_provider import AuthTokenProvider, AuthTokenProviderFactory
from .simple_auth_token_provider import SimpleAuthTokenProvider, SimpleAuthTokenProviderFactory
from .twitter_auth_token_provider import TwitterAuthTokenProvider, TwitterAuthTokenProviderFactory
